C+++
C
C FACILITY
C	OAK RIDGE NATIONAL LABORATORY
C	FUSION ENERGY DIVISION
C	TOKAMAK SECTION
C
C ABSTRACT
C	This FORTRAN procedure library provides several support procedures
C	for the FED VAX CAMAC system. The procedures included presently fall
C	into one of two categories, namely (1) access to privileged CAMAC
C	ACP functions and (2) CAMAC status display. Specifically, the library
C	contains the following procedures:
C
C	CAM$CONL
C	-Request ACP set CAMAC crate unit 'ON-LINE',
C	CAM$ENLAMS
C	-Request ACP to enable LAM support for a specified CAMAC
C	 crate unit,
C	CAM$COFL
C	-Request ACP to set a specified CAMAC crate unit 'OFF-LINE',
C	CAM$DABLAMS
C	-Request ACP to disable LAM support for a specified CAMAC
C	 crate unit,
C
C AUTHOR
C	E.T. Blair
C
C REVISION HISTORY
C	V01-001,E.T. Blair,25-AUG-1982
C	Added documentation for internal procedure CAM_SUPINIT.
C
C	Modified CAM$CONL, CAM$COFL, CAM$ENLAMS, CAM$DABLAMS procedures
C	to return the CAMAC IOSB resulting from QIO's to the CAMAC ACP
C	to the calling program. These procedures no longer try to interpret
C	the CAMAC IOSB for success/error indication. This now becomes the
C	responsibility of the caller, just as for the standard CAMAC access
C	calls.
C
C---
C+++
C
C	Procedure:
C	CAM$CONL(UNIT_NAME,IOSB)
C
C	Description:
C	This procedure makes a request to the CAMAC ACP to set the
C	specified CAMAC crate unit 'ON-LINE'. The ACP performs the
C	necessary CAMAC commands to bring the CAMAC crate controller 
C	to an operational state and performs the necessary internal 
C	data structure manipulation to set the crate UCB 'ON-LINE'.
C
C	Parameters:
C	UNIT_NAME- address of a CHARACTER string descriptor specifying
C		an ASCII string identifying the crate unit to be set
C		on-line.
C	IOSB- address of a quadword buffer to be used as the CAMAC I/O
C		status block for the I/O request.
C
C	Operational Notes:
C	1. Successful execution of this procedure requires that the
C	   process executing the image using CAM$CONL have PHYS_IO
C	   privilege.
C
C
C	Return Status:
C	SS$_NORMAL- CAMAC request successful,
C	Any other system status indicates faliure(see system
C		status message definitions for explicit error
C		message).
C
C---
	INTEGER FUNCTION CAM$CONL(UNIT_NAME,IOSB)
	IMPLICIT INTEGER(A-Z)
	EXTERNAL IO$_SETCHAR,IO$M_CONLINE
	INTEGER*4 IOSB(*)
	CHARACTER UNIT_NAME*(*),HSU_NAME*6
	INTEGER*2 HSU_CHAN
C++
C
C	CAM$CONL INITIALIZATION
C
C--
	IOSB(1)=0
	IOSB(2)=0
C++
C
C	DECODE SPECIFIED UNIT AND ASSIGN CHANNEL TO ASSOCIATED HSU.
C
C--
	CAM$CONL=CAM$CSU(UNIT_NAME,HSU_NAME,CRATE_NUMBER)
	IF (.NOT.CAM$CONL) THEN
		RETURN
	END IF
C++
C
C	ASSIGN I/O CHANNEL TO CAMAC HIGHWAY UNIT
C
C--
	CAM$CONL=SYS$ASSIGN(HSU_NAME,HSU_CHAN,,)
	IF (.NOT.CAM$CONL) THEN
		RETURN
	END IF
C++
C
C	QIO TO CAMAC ACP TO SET CRATE ON-LINE
C
C--
	CAM$CONL=SYS$QIOW(%VAL(0),%VAL(HSU_CHAN),
	9	%VAL(%LOC(IO$_SETCHAR)+%LOC(IO$M_CONLINE)),
	9	IOSB,,,%VAL(CRATE_NUMBER),,,,,)
D	TYPE 2999,CAM$CONL,IOSB(1),IOSB(2)
D2999	FORMAT(1X,'R0=',Z8,' IOSB=',Z8,2X,Z8)
C++
C
C	RELEASE VMS I/O CHANNEL ASSIGNMENT
C
C--
	CALL SYS$DASSGN(%VAL(HSU_CHAN))
	RETURN
	END
C+++
C
C	Procedure:
C	CAM$ENLAMS(UNIT_NAME,LAMG_N,IOSB)
C
C	Description:
C	This procedure makes a request to the CAMAC ACP to enable LAM
C	support for the specified CAMAC crate unit. The ACP performs the
C	necessary CAMAC commands to bring the CAMAC crate controller 
C	and LAM grader/encoder to a state capable of generating CAMAC
C	demands to the associated VAX highway driver. The ACP also allocates
C	and initializes the necessary internal data structures to allow
C	LAM AST requests to be queued to specific stations within the
C	crate. Each crate is considered to have a total of 24 LAM stations
C	numbered 0-23, where station 0 corresponds to the internal LAM
C	associated with serial CAMAC L2 crate controllers. Attempts to
C	perform LAM service requests to a crate without LAM support enabled
C	will result in failure status from the CAMAC system.
C
C	Parameters:
C	UNIT_NAME- address of a CHARACTER string descriptor specifying
C		an ASCII string identifying the crate unit to be set
C		on-line.
C	LAMG_N- value of the CAMAC station to be used to address the
C		LAM grader/encoder module.
C	IOSB- address of a quadword buffer to be used as the CAMAC I/O
C		status block for the I/O request.
C
C	Operational Notes:
C	1. Successful execution of this procedure requires that the
C	   process executing the image using CAM$ENLAMS have PHYS_IO
C	   privilege.
C	2. The VAX CAMAC LAM management philosophy assumes the use of
C	   LAM grader/encoder modules to provide application independent
C	   control over the generation of CAMAC demands. In this way, system
C	   integrity is not compromised by a failure to clear an existing LAM.
C
C
C	Return Status:
C	SS$_NORMAL- CAMAC request successful,
C	Any other system status indicates faliure(see system
C		status message definitions for explicit error
C		message).
C
C---
	INTEGER FUNCTION CAM$ENLAMS(UNIT_NAME,LAMG_N,IOSB)
	IMPLICIT INTEGER(A-Z)
	EXTERNAL IO$_SETCHAR,IO$M_ENLAMS
	INTEGER*4 IOSB(*)
	CHARACTERUNIT_NAME*(*),HSU_NAME*6
	INTEGER*2 HSU_CHAN
C++
C
C	CAM$ENLAMS INITIALIZATION
C
C--
	IOSB(1)=0
	IOSB(2)=0
C++
C
C	DECODE SPECIFIED UNIT AND ASSIGN CHANNEL TO ASSOCIATED HSU.
C
C--
	CAM$ENLAMS=CAM$CSU(UNIT_NAME,HSU_NAME,CRATE_NUMBER)
	IF (.NOT.CAM$ENLAMS) THEN
		RETURN
	END IF
C++
C
C	ASSIGN I/O CHANNEL TO CAMAC HIGHWAY UNIT
C
C--
	CAM$ENLAMS=SYS$ASSIGN(HSU_NAME,HSU_CHAN,,)
	IF (.NOT.CAM$ENLAMS) THEN
		RETURN
	END IF
C++
C
C	QIO TO CAMAC ACP TO ENABLE LAM SUPPORT FOR CRATE
C
C--
	CAM$ENLAMS=SYS$QIOW(%VAL(0),%VAL(HSU_CHAN),
	9	%VAL(%LOC(IO$_SETCHAR)+%LOC(IO$M_ENLAMS)),
	9	IOSB,,,%VAL(CRATE_NUMBER),LAMG_N,,,,)
D	TYPE 2999,CAM$ENLAMS,IOSB(1),IOSB(2)
D2999	FORMAT(1X,'R0=',Z8,' IOSB=',Z8,2X,Z8)
C++
C
C	RELEASE VMS I/O CHANNEL ASSIGNMENT
C
C--
	CALL SYS$DASSGN(%VAL(HSU_CHAN))
	RETURN
	END
C+++
C
C	Procedure:
C	CAM$COFL(UNIT_NAME,IOSB)
C
C	Description:
C	This procedure makes a request to the CAMAC ACP to set the
C	specified CAMAC crate unit 'OFF-LINE'. The ACP performs the
C	necessary CAMAC commands to disable further CAMAC crate controller
C	activity and performs the necessary internal data structure
C	manipulation to set the crate UCB 'OFF-LINE'.
C
C	Parameters:
C	UNIT_NAME- address of a CHARACTER string descriptor specifying
C		an ASCII string identifying the crate unit to be set
C		on-line.
C	IOSB- address of a quadword buffer to be used as the CAMAC I/O
C		status block for the I/O request.
C
C	Operational Notes:
C	1. Successful execution of this procedure requires that the
C	   process executing the image using CAM$COFL have PHYS_IO
C	   privilege.
C
C
C	Return Status:
C	SS$_NORMAL- CAMAC request successful,
C	Any other system status indicates faliure(see system
C		status message definitions for explicit error
C		message).
C
C---
	INTEGER FUNCTION CAM$COFL(UNIT_NAME,IOSB)
	IMPLICIT INTEGER(A-Z)
	EXTERNAL IO$_SETCHAR,IO$M_COFFLINE
	INTEGER*4 IOSB(*)
	CHARACTERUNIT_NAME*(*),HSU_NAME*6
	INTEGER*2 HSU_CHAN
C++
C
C	CAM$COFL INITIALIZATION
C
C--
	IOSB(1)=0
	IOSB(2)=0
C++
C
C	DECODE SPECIFIED UNIT AND ASSIGN CHANNEL TO HSU.
C
C--
	CAM$COFL=CAM$CSU(UNIT_NAME,HSU_NAME,CRATE_NUMBER)
	IF (.NOT.CAM$COFL) THEN
		RETURN
	END IF
C++
C
C	ASSIGN I/O CHANNEL TO CAMAC HIGHWAY UNIT
C
C--
	CAM$COFL=SYS$ASSIGN(HSU_NAME,HSU_CHAN,,)
	IF (.NOT.CAM$COFL) THEN
		RETURN
	END IF
C++
C
C	QIO TO CAMAC ACP TO SET CRATE OFF-LINE.
C
C--
	CAM$COFL=SYS$QIOW(%VAL(0),%VAL(HSU_CHAN),
	9	%VAL(%LOC(IO$_SETCHAR)+%LOC(IO$M_COFFLINE)),
	9	IOSB,,,%VAL(CRATE_NUMBER),,,,,)
D	TYPE 2999,CAM$COFL,IOSB(1),IOSB(2)
D2999	FORMAT(1X,'R0=',Z8,' IOSB=',Z8,2X,Z8)
C++
C
C	RELEASE VMS I/O CHANNEL ASSIGNMENT
C
C--
	CALL SYS$DASSGN(%VAL(HSU_CHAN))
	RETURN
	END
C+++
C
C	Procedure:
C	CAM$DABLAMS(UNIT_NAME,IOSB)
C
C	Description:
C	This procedure makes a request to the CAMAC ACP to disable LAM
C	support for the specified CAMAC crate unit. The ACP performs the
C	necessary CAMAC commands to take the CAMAC crate controller 
C	and LAM grader/encoder to a state incapable of generating CAMAC
C	demands to the associated VAX highway driver. The ACP then flushes
C	all pending AST requests for the specified CAMAC crate, accounting
C	for the AST charged aganist the process in the process AST quota.
C	Then the associated internal data structure used by the ACP for
C	LAM management support is deallocated back to non-paged pool.
C
C	Parameters:
C	UNIT_NAME- address of a CHARACTER string descriptor specifying
C		an ASCII string identifying the crate unit to be set
C		on-line.
C	IOSB- address of a quadword buffer to be used as the CAMAC I/O
C		status block for the I/O request.
C
C	Operational Notes:
C	1. Successful execution of this procedure requires that the
C	   process executing the image using CAM$DABLAMS have PHYS_IO
C	   privilege.
C	2. The VAX CAMAC LAM management philosophy assumes the use of
C	   LAM grader/encoder modules to provide application independent
C	   control over the generation of CAMAC demands. In this way, system
C	   integrity is not compromised by a failure to clear an existing LAM.
C
C
C	Return Status:
C	SS$_NORMAL- CAMAC request successful,
C	Any other system status indicates faliure(see system
C		status message definitions for explicit error
C		message).
C
C---
	INTEGER FUNCTION CAM$DABLAMS(UNIT_NAME,IOSB)
	IMPLICIT INTEGER(A-Z)
	EXTERNAL IO$_SETCHAR,IO$M_DABLAMS
	INTEGER*4 IOSB(*)
	CHARACTER UNIT_NAME*(*),HSU_NAME*6
	INTEGER*2 HSU_CHAN
C++
C
C	CAM$DABLAMS INITIALIZATION
C
C--
	IOSB(1)=0
	IOSB(2)=0
C++
C
C	DECODE SPECIFIED UNIT AND ASSIGN CHANNEL TO ASSOCIATED HSU.
C
C--
	CAM$DABLAMS=CAM$CSU(UNIT_NAME,HSU_NAME,CRATE_NUMBER)
	IF (.NOT.CAM$DABLAMS) THEN
		RETURN
	END IF
C++
C
C	ASSIGN I/O CHANNEL TO CAMAC HIGHWAY UNIT
C
C--
	CAM$DABLAMS=SYS$ASSIGN(HSU_NAME,HSU_CHAN,,)
	IF (.NOT.CAM$DABLAMS) THEN
		RETURN
	END IF
C++
C
C	QIO TO CAMAC ACP TO DISABLE LAM SUPPORT FOR CRATE.
C
C--
	CAM$DABLAMS=SYS$QIOW(%VAL(0),%VAL(HSU_CHAN),
	9	%VAL(%LOC(IO$_SETCHAR)+%LOC(IO$M_DABLAMS)),
	9	IOSB,,,%VAL(CRATE_NUMBER),,,,,)
D	TYPE 2999,CAM$DABLAMS,IOSB(1),IOSB(2)
D2999	FORMAT(1X,'R0=',Z8,' IOSB=',Z8,2X,Z8)
C++
C
C	RELEASE VMS I/O CHANNEL ASSIGNMENT
C
C--
	CALL SYS$DASSGN(%VAL(HSU_CHAN))
	RETURN
	END
C+++
C
C	Procedure:
C	CAM$CSU(UNIT_NAME,HSU_NAME,CRATE_NUMBER)
C
C	Description:
C	This procedure parses the CAMAC Crate Support Unit name input
C	in the CHARACTER variable UNIT_NAME, returning the name of the
C	CAMAC Highway Support Unit for the CSU and the crate number
C	from the CSU.
C
C	Parameters:
C	UNIT_NAME- address of a descriptor for a CHARACTER variable containing
C	     a valid CAMAC Crate Support Unit name,
C	HSU_NAME- address of a descriptor for a CHARACTER variable to receive
C	     the Highway Support Unit name for the input CSU,
C	CRATE_NUMBER- address of a longword to receive the CAMAC crate number
C	     taken from the CSU name.
C
C---
	INTEGER FUNCTION CAM$CSU(UNIT_NAME,HSU_NAME,CRATE_NUMBER)
	IMPLICIT INTEGER(A-Z)
	EXTERNAL SS$_NORMAL,SS$_IVDEVNAM
	CHARACTER UNIT_NAME*(*),HSU_NAME*(*)
C++
C	ASSUME SUCCESS
C
C--
	CAM$CSU=%LOC(SS$_NORMAL)
C++
C
C	VERIFY CSU PREFIX
C
C--
	IF (UNIT_NAME(1:2).NE.'SH' .AND.
	9	UNIT_NAME(1:2).NE.'PH') THEN
		CAM$CSU=%LOC(SS$_IVDEVNAM)
		RETURN
	END IF
C++
C
C	EXAMINE UNIT MNEMONIC FOR CRATE NUMBER
C
C--
	COLON_POS=INDEX(UNIT_NAME,':')
	IF (COLON_POS.NE.LEN(UNIT_NAME) .OR. 
	9	COLON_POS.LT.5) THEN
		CAM$CSU=%LOC(SS$_IVDEVNAM)
		RETURN
	END IF
	DECODE(COLON_POS-4,1999,UNIT_NAME(4:)) CRATE_NUMBER
1999	FORMAT(I)
C++
C
C	CONSTRUCT CAMAC HIGHWAY SUPPORT UNIT NAME FROM
C	SPECIFIED UNIT NAME.
C
C--
	HSU_NAME(1:1)='_'
	HSU_NAME(2:4)=UNIT_NAME(1:3)
	HSU_NAME(5:6)='0:'
	RETURN
	END
